package inf101.v18.rogue101.events;

import inf101.v18.rogue101.game.Game;
import inf101.v18.rogue101.object.Item;

/**
 * Example implementation of events – could be used to have more complex
 * behaviour than just attack/defend/get damaged/pickup/drop.
 *
 * @param <T> Relevant extra data for this particular event
 * @author anya
 */
public class GameEvent<T> implements IEvent<T> {

    private final String name;
    private final Item source;
    private final Item target;
    private T value;

    private final Game game;

    /**
     * Create a new game event
     *
     * @param name   The name is used when checking which event this is / determine its
     *               “meaning”
     * @param game   The game, or <code>null</code> if unknown/not relevant
     * @param source The item that caused the event, or <code>null</code> if
     *               unknown/not relevant
     * @param target The item that receives the event, or <code>null</code> if
     *               unknown/not relevant
     * @param value  Arbitrary extra data
     */
    public GameEvent(String name, Game game, Item source, Item target, T value) {
        this.name = name;
        this.game = game;
        this.source = source;
        this.target = target;
        this.value = value;
    }

    /**
     * Create a new game event
     *
     * @param name   The name is used when checking which event this is / determine its
     *               “meaning”
     * @param source The item that caused the event, or <code>null</code> if
     *               unknown/not relevant
     */
    public GameEvent(String name, Item source) {
        this(name, null, source, null, null);
    }

    /**
     * Create a new game event
     *
     * @param name   The name is used when checking which event this is / determine its
     *               “meaning”
     * @param source The item that caused the event, or <code>null</code> if
     *               unknown/not relevant
     * @param value  Arbitrary extra data
     */
    public GameEvent(String name, Item source, T value) {
        this(name, null, source, null, value);
    }

    @Override
    public T getData() {
        return value;
    }

    @Override
    public String getEventName() {
        return name;
    }

    @Override
    public Game getGame() {
        return game;
    }

    @Override
    public Item getSource() {
        return source;
    }

    @Override
    public Item getTarget() {
        return target;
    }

    @Override
    public void setData(T value) {
        this.value = value;
    }

}
