package inf101.v18.rogue101.items.container;

import inf101.v18.rogue101.game.Game;
import inf101.v18.rogue101.items.buff.Binoculars;
import inf101.v18.rogue101.items.buff.Shield;
import inf101.v18.rogue101.items.consumable.HealthPotion;
import inf101.v18.rogue101.items.consumable.Manga;
import inf101.v18.rogue101.items.weapon.BasicBow;
import inf101.v18.rogue101.items.weapon.BasicSword;
import inf101.v18.rogue101.items.weapon.FireStaff;
import inf101.v18.rogue101.object.Item;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

public class Chest implements Container, Static {
    private final List<Item> container;
    private final int MAX_SIZE = 10;

    public Chest() {
        this.container = new ArrayList<>();
    }

    /**
     * Randomly fills chest with random items based on dungeon level.
     *
     * @param lvl The current dungeon level
     */
    public void fill(int lvl) {
        Random random = new Random();
        int itemChance = 3;
        List<Item> items = new ArrayList<>();
        items.add(new FireStaff());
        items.add(new BasicSword());
        items.add(new BasicBow());
        items.add(new Binoculars());
        items.add(new Shield());
        items.add(new Manga());
        for (int i = 0; i < MAX_SIZE; i++) {
            int num = random.nextInt(100);
            boolean added = false;
            for (int j = 0; j < items.size(); j++) {
                if (num < (itemChance * (j + 1)) + ((j + 1) * lvl)) {
                    addItem(items.get(j));
                    items.remove(j); //We don't want duplicates
                    added = true;
                    break;
                }
            }
            if (!added && num > 70) {
                addItem(new HealthPotion());
            }
        }
    }

    @Override
    public Item get(int i) {
        return container.get(i);
    }

    @Override
    public List<Item> getContent() {
        return Collections.unmodifiableList(container);
    }

    @Override
    public boolean hasSpace() {
        return container.size() < MAX_SIZE;
    }

    @Override
    public Item getFirst(Class<?> clazz) {
        for (Item item : container) {
            if (clazz.isInstance(item)) {
                return item;
            }
        }
        return null;
    }

    @Override
    public int getCurrentHealth() {
        return 0;
    }

    @Override
    public int getMaxHealth() {
        return 0;
    }

    @Override
    public String getName() {
        return "Chest";
    }

    @Override
    public int getSize() {
        return 10000;
    }

    public String getPrintSymbol() {
        return "\u001b[94m" + "\uD83D\uDDC3" + "\u001b[0m";
    }

    @Override
    public String getSymbol() {
        return "C";
    }

    @Override
    public int handleDamage(Game game, Item source, int amount) {
        return 0;
    }

    @Override
    public boolean addItem(Item item) {
        if (container.size() < MAX_SIZE) {
            container.add(item);
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void remove(int i) {
        container.remove(i);
    }
}
