package inf101.v18.rogue101.object;

import inf101.v18.rogue101.game.Game;

/**
 * An actor is an Item that can also do something, either controlled by the
 * computer (NonPlayerCharacter) or the user (PlayerCharacter).
 *
 * @author anya
 */
public interface Actor extends Item {

    /**
     * @return This actor's attack score (used against an item's
     * {@link #getDefense()} score to see if an attack is successful)
     */
    int getAttack();

    /**
     * @return The damage this actor deals on a successful attack (used together
     * with
     * {@link #handleDamage(Game, Item, int)} on
     * the target)
     */
    int getDamage();

    /**
     * How many tiles the IActor is able to see in each direction.
     *
     * @return Number of tiles
     */
    default int getVision() {
        return 1;
    }

    /**
     * Gets an item of the specified type, if the actor has the item.
     *
     * @return An IItem or null
     */
    Item getItem(Class<?> type);

}
