package inf101.v18.rogue101.state;

import inf101.v18.util.NPCHelper;
import javafx.scene.media.AudioClip;

import java.net.URL;

/**
 * A sound that can be played
 */
public enum Sound {

    MELEE_NO_WEAPON("Realistic_Punch-Mark_DiAngelo-1609462330.wav"),
    MELEE_SWORD("Sword Swing-SoundBible.com-639083727.wav"),
    RANGED_NO_WEAPON("Snow Ball Throw And Splat-SoundBible.com-992042947.wav"),
    RANGED_BOW("Bow_Fire_Arrow-Stephan_Schutze-2133929391.wav"),
    RANGED_STAFF("Large Fireball-SoundBible.com-301502490.wav"),
    GAME_OVER("Dying-SoundBible.com-1255481835.wav"),
    WIN("1_person_cheering-Jett_Rifkin-1851518140.wav"),
    NPC_DEATH("oh-no-113125.wav"),
    ;

    private final String soundFile;
    private AudioClip audioClip = null;

    /**
     * Instantiates a new sound file
     *
     * @param soundFile <p>The file containing this sound</p>
     */
    Sound(String soundFile) {
        this.soundFile = soundFile;
    }

    /**
     * Gets the sound file for this sound
     *
     * @return <p>The sound file for this sound</p>
     */
    public String getSoundFile() {
        return "audio/" + soundFile;
    }

    /**
     * Plays this sound
     */
    public void play() {
        if (audioClip == null) {
            URL url = NPCHelper.class.getClassLoader().getResource(getSoundFile());
            if (url != null) {
                audioClip = new AudioClip(url.toString());
            } else {
                System.out.println("Could not play audio file " + getSoundFile());
            }
        }

        // Play the sound in a new thread if it could be loaded
        if (audioClip != null) {
            new Thread(() -> audioClip.play()).start();
        }
    }

}
